<!-- templates/index.html -->

{% extends "base.html" %}

{% block content %}
<h1 class="title">
  KuttyPy Compiler
</h1>
<h2 class="subtitle">
  Just a landing page. POST C files to the /compile endpoint
</h2>
{% endblock %}
