# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ipy.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(847, 411)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.centralwidget)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.frame = QtWidgets.QFrame(self.centralwidget)
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.layout = QtWidgets.QVBoxLayout(self.frame)
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.layout.setSpacing(0)
        self.layout.setObjectName("layout")
        self.verticalLayout.addWidget(self.frame)
        MainWindow.setCentralWidget(self.centralwidget)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.toolBar = QtWidgets.QToolBar(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.toolBar.sizePolicy().hasHeightForWidth())
        self.toolBar.setSizePolicy(sizePolicy)
        self.toolBar.setObjectName("toolBar")
        MainWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.toolBar)
        self.actionNumpy = QtWidgets.QAction(MainWindow)
        self.actionNumpy.setObjectName("actionNumpy")
        self.actionPylab = QtWidgets.QAction(MainWindow)
        self.actionPylab.setObjectName("actionPylab")
        self.actionScipy = QtWidgets.QAction(MainWindow)
        self.actionScipy.setObjectName("actionScipy")
        self.toolBar.addAction(self.actionNumpy)
        self.toolBar.addAction(self.actionScipy)
        self.toolBar.addAction(self.actionPylab)

        self.retranslateUi(MainWindow)
        self.actionNumpy.triggered.connect(MainWindow.importNumpy) # type: ignore
        self.actionScipy.triggered.connect(MainWindow.importScipy) # type: ignore
        self.actionPylab.triggered.connect(MainWindow.importPylab) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "iPython Console"))
        self.toolBar.setWindowTitle(_translate("MainWindow", "toolBar"))
        self.actionNumpy.setText(_translate("MainWindow", "Numpy"))
        self.actionNumpy.setToolTip(_translate("MainWindow", "import numpy as np"))
        self.actionNumpy.setShortcut(_translate("MainWindow", "Ctrl+Shift+N"))
        self.actionPylab.setText(_translate("MainWindow", "PyLab"))
        self.actionPylab.setToolTip(_translate("MainWindow", "from pylab import *"))
        self.actionPylab.setShortcut(_translate("MainWindow", "Ctrl+Shift+P"))
        self.actionScipy.setText(_translate("MainWindow", "Scipy"))
        self.actionScipy.setToolTip(_translate("MainWindow", "import scipy"))
        self.actionScipy.setShortcut(_translate("MainWindow", "Ctrl+Shift+S"))
