# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'layoutuno.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(754, 635)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/control/kuttypy.png"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        MainWindow.setWindowIcon(icon)
        MainWindow.setStyleSheet("")
        MainWindow.setDockOptions(QtWidgets.QMainWindow.AnimatedDocks)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout_2.setSizeConstraint(QtWidgets.QLayout.SetMaximumSize)
        self.gridLayout_2.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_2.setSpacing(0)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.tabs = QtWidgets.QTabWidget(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tabs.sizePolicy().hasHeightForWidth())
        self.tabs.setSizePolicy(sizePolicy)
        self.tabs.setObjectName("tabs")
        self.playground = QtWidgets.QWidget()
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.playground.sizePolicy().hasHeightForWidth())
        self.playground.setSizePolicy(sizePolicy)
        self.playground.setObjectName("playground")
        self.gridLayout = QtWidgets.QGridLayout(self.playground)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtWidgets.QLabel(self.playground)
        self.label.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.label.setScaledContents(True)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 1, 1, 1)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem, 0, 0, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(64, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout.addItem(spacerItem1, 0, 2, 1, 1)
        self.tabs.addTab(self.playground, "")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.tab)
        self.gridLayout_4.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_4.setSpacing(0)
        self.gridLayout_4.setObjectName("gridLayout_4")
        spacerItem2 = QtWidgets.QSpacerItem(51, 24, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.gridLayout_4.addItem(spacerItem2, 0, 1, 1, 1)
        self.registerList = QtWidgets.QListWidget(self.tab)
        self.registerList.setDragDropMode(QtWidgets.QAbstractItemView.InternalMove)
        self.registerList.setDefaultDropAction(QtCore.Qt.MoveAction)
        self.registerList.setObjectName("registerList")
        self.gridLayout_4.addWidget(self.registerList, 1, 0, 1, 3)
        self.pushButton = QtWidgets.QPushButton(self.tab)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/control/plus.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton.setIcon(icon1)
        self.pushButton.setObjectName("pushButton")
        self.gridLayout_4.addWidget(self.pushButton, 0, 0, 1, 1)
        self.autoRefreshUserRegisters = QtWidgets.QCheckBox(self.tab)
        self.autoRefreshUserRegisters.setObjectName("autoRefreshUserRegisters")
        self.gridLayout_4.addWidget(self.autoRefreshUserRegisters, 0, 2, 1, 1)
        self.tabs.addTab(self.tab, "")
        self.examples = QtWidgets.QWidget()
        self.examples.setObjectName("examples")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.examples)
        self.gridLayout_3.setContentsMargins(0, 0, 0, 0)
        self.gridLayout_3.setHorizontalSpacing(3)
        self.gridLayout_3.setVerticalSpacing(2)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.toolButton = QtWidgets.QToolButton(self.examples)
        self.toolButton.setText("")
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/control/play.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.toolButton.setIcon(icon2)
        self.toolButton.setObjectName("toolButton")
        self.gridLayout_3.addWidget(self.toolButton, 0, 2, 1, 1)
        self.exampleList = QtWidgets.QComboBox(self.examples)
        self.exampleList.setObjectName("exampleList")
        self.gridLayout_3.addWidget(self.exampleList, 0, 0, 1, 1)
        self.userCode = QtWidgets.QPlainTextEdit(self.examples)
        font = QtGui.QFont()
        font.setPointSize(11)
        self.userCode.setFont(font)
        self.userCode.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.userCode.setLineWrapMode(QtWidgets.QPlainTextEdit.NoWrap)
        self.userCode.setTabStopWidth(30)
        self.userCode.setBackgroundVisible(False)
        self.userCode.setCenterOnScroll(False)
        self.userCode.setProperty("class", "")
        self.userCode.setObjectName("userCode")
        self.gridLayout_3.addWidget(self.userCode, 1, 0, 1, 3)
        self.toolButton_2 = QtWidgets.QToolButton(self.examples)
        self.toolButton_2.setText("")
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/control/stop.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.toolButton_2.setIcon(icon3)
        self.toolButton_2.setObjectName("toolButton_2")
        self.gridLayout_3.addWidget(self.toolButton_2, 0, 1, 1, 1)
        self.pushButton_2 = QtWidgets.QPushButton(self.examples)
        self.pushButton_2.setObjectName("pushButton_2")
        self.gridLayout_3.addWidget(self.pushButton_2, 2, 0, 1, 3)
        self.tabs.addTab(self.examples, "")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.tab_2)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.filenameLabel = QtWidgets.QLabel(self.tab_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.filenameLabel.sizePolicy().hasHeightForWidth())
        self.filenameLabel.setSizePolicy(sizePolicy)
        self.filenameLabel.setObjectName("filenameLabel")
        self.gridLayout_5.addWidget(self.filenameLabel, 0, 1, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_5.addItem(spacerItem3, 2, 0, 1, 1)
        self.commandLinkButton_2 = QtWidgets.QCommandLinkButton(self.tab_2)
        self.commandLinkButton_2.setObjectName("commandLinkButton_2")
        self.gridLayout_5.addWidget(self.commandLinkButton_2, 0, 0, 1, 1)
        self.commandLinkButton_3 = QtWidgets.QCommandLinkButton(self.tab_2)
        self.commandLinkButton_3.setObjectName("commandLinkButton_3")
        self.gridLayout_5.addWidget(self.commandLinkButton_3, 1, 0, 1, 1)
        self.tabs.addTab(self.tab_2, "")
        self.gridLayout_2.addWidget(self.tabs, 0, 0, 1, 6)
        self.enableLog = QtWidgets.QCheckBox(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.enableLog.sizePolicy().hasHeightForWidth())
        self.enableLog.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(9)
        font.setItalic(True)
        self.enableLog.setFont(font)
        self.enableLog.setChecked(True)
        self.enableLog.setObjectName("enableLog")
        self.gridLayout_2.addWidget(self.enableLog, 6, 0, 1, 1)
        self.log = QtWidgets.QTextBrowser(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.log.sizePolicy().hasHeightForWidth())
        self.log.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(9)
        self.log.setFont(font)
        self.log.setLineWrapMode(QtWidgets.QTextEdit.WidgetWidth)
        self.log.setObjectName("log")
        self.gridLayout_2.addWidget(self.log, 3, 0, 2, 6)
        self.logLabel = QtWidgets.QLabel(self.centralwidget)
        self.logLabel.setMinimumSize(QtCore.QSize(0, 30))
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setItalic(True)
        self.logLabel.setFont(font)
        self.logLabel.setAlignment(QtCore.Qt.AlignBottom|QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft)
        self.logLabel.setObjectName("logLabel")
        self.gridLayout_2.addWidget(self.logLabel, 2, 0, 1, 6)
        self.clearLog = QtWidgets.QCheckBox(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.clearLog.sizePolicy().hasHeightForWidth())
        self.clearLog.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(9)
        font.setItalic(True)
        self.clearLog.setFont(font)
        self.clearLog.setChecked(True)
        self.clearLog.setObjectName("clearLog")
        self.gridLayout_2.addWidget(self.clearLog, 6, 1, 1, 1)
        self.toolButton_4 = QtWidgets.QToolButton(self.centralwidget)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/control/refresh.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.toolButton_4.setIcon(icon4)
        self.toolButton_4.setObjectName("toolButton_4")
        self.gridLayout_2.addWidget(self.toolButton_4, 6, 5, 1, 1)
        self.frame = QtWidgets.QFrame(self.centralwidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.frame.sizePolicy().hasHeightForWidth())
        self.frame.setSizePolicy(sizePolicy)
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.sensorLayout = QtWidgets.QHBoxLayout(self.frame)
        self.sensorLayout.setContentsMargins(0, 0, 0, 0)
        self.sensorLayout.setSpacing(2)
        self.sensorLayout.setObjectName("sensorLayout")
        self.gridLayout_2.addWidget(self.frame, 6, 2, 1, 3)
        MainWindow.setCentralWidget(self.centralwidget)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.leftdock = QtWidgets.QDockWidget(MainWindow)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.leftdock.sizePolicy().hasHeightForWidth())
        self.leftdock.setSizePolicy(sizePolicy)
        self.leftdock.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.leftdock.setAllowedAreas(QtCore.Qt.LeftDockWidgetArea|QtCore.Qt.TopDockWidgetArea)
        self.leftdock.setWindowTitle("")
        self.leftdock.setObjectName("leftdock")
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.leftlayout = QtWidgets.QVBoxLayout(self.dockWidgetContents)
        self.leftlayout.setObjectName("leftlayout")
        self.leftdock.setWidget(self.dockWidgetContents)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.leftdock)
        self.rightdock = QtWidgets.QDockWidget(MainWindow)
        self.rightdock.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.rightdock.setAllowedAreas(QtCore.Qt.RightDockWidgetArea|QtCore.Qt.TopDockWidgetArea)
        self.rightdock.setWindowTitle("")
        self.rightdock.setObjectName("rightdock")
        self.dockWidgetContents_5 = QtWidgets.QWidget()
        self.dockWidgetContents_5.setObjectName("dockWidgetContents_5")
        self.rightlayout = QtWidgets.QVBoxLayout(self.dockWidgetContents_5)
        self.rightlayout.setObjectName("rightlayout")
        self.rightdock.setWidget(self.dockWidgetContents_5)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(2), self.rightdock)

        self.retranslateUi(MainWindow)
        self.tabs.setCurrentIndex(0)
        self.toolButton.clicked.connect(MainWindow.runCode) # type: ignore
        self.tabs.currentChanged['int'].connect(MainWindow.tabChanged) # type: ignore
        self.toolButton_2.clicked.connect(MainWindow.abort) # type: ignore
        self.exampleList.currentIndexChanged['QString'].connect(MainWindow.loadExample) # type: ignore
        self.pushButton.clicked.connect(MainWindow.newRegister) # type: ignore
        self.autoRefreshUserRegisters.clicked['bool'].connect(MainWindow.userRegistersAutoRefresh) # type: ignore
        self.commandLinkButton_2.clicked.connect(MainWindow.openFile) # type: ignore
        self.commandLinkButton_3.clicked.connect(MainWindow.compileAndUpload) # type: ignore
        self.toolButton_4.clicked.connect(MainWindow.I2CScan) # type: ignore
        self.pushButton_2.clicked.connect(MainWindow.ipython) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "KuttyPy Interactive Console"))
        self.label.setText(_translate("MainWindow", "<html><head/><body><p><img src=\":/control/unopinout.png\"/></p></body></html>"))
        self.tabs.setTabText(self.tabs.indexOf(self.playground), _translate("MainWindow", "Playground"))
        self.pushButton.setText(_translate("MainWindow", "Add A New Register"))
        self.autoRefreshUserRegisters.setText(_translate("MainWindow", "Auto-Refresh"))
        self.tabs.setTabText(self.tabs.indexOf(self.tab), _translate("MainWindow", "Registers"))
        self.userCode.setPlainText(_translate("MainWindow", "import time\n"
"from kuttyPy import *\n"
"setReg(DDRD,255)\n"
"\n"
"for a in range(5):   #Run this loop 5 times. \n"
"    setReg(PORTD, 255)\n"
"    time.sleep(0.5)\n"
"    setReg(PORTD, 0)\n"
"    time.sleep(0.5)"))
        self.pushButton_2.setText(_translate("MainWindow", "Launch iPython Console"))
        self.tabs.setTabText(self.tabs.indexOf(self.examples), _translate("MainWindow", "Scripting"))
        self.filenameLabel.setText(_translate("MainWindow", ":"))
        self.commandLinkButton_2.setText(_translate("MainWindow", "Open File"))
        self.commandLinkButton_3.setText(_translate("MainWindow", "Compile+Upload"))
        self.tabs.setTabText(self.tabs.indexOf(self.tab_2), _translate("MainWindow", "C Code"))
        self.enableLog.setText(_translate("MainWindow", "Enabled"))
        self.logLabel.setText(_translate("MainWindow", "Monitor registers being read and set during each operation"))
        self.clearLog.setText(_translate("MainWindow", "Auto-Clear"))
        self.toolButton_4.setText(_translate("MainWindow", "..."))
from . import uno_rc
