# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'dio_cntr.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Frame(object):
    def setupUi(self, Frame):
        Frame.setObjectName("Frame")
        Frame.resize(292, 58)
        Frame.setFrameShape(QtWidgets.QFrame.NoFrame)
        Frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.horizontalLayout = QtWidgets.QHBoxLayout(Frame)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setSpacing(0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.stack = QtWidgets.QStackedWidget(Frame)
        self.stack.setObjectName("stack")
        self.page = QtWidgets.QWidget()
        self.page.setObjectName("page")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.page)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_2.setSpacing(0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.nameIn = QtWidgets.QCheckBox(self.page)
        self.nameIn.setObjectName("nameIn")
        self.horizontalLayout_2.addWidget(self.nameIn)
        self.pullup = QtWidgets.QCheckBox(self.page)
        self.pullup.setObjectName("pullup")
        self.horizontalLayout_2.addWidget(self.pullup)
        spacerItem = QtWidgets.QSpacerItem(57, 26, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.stack.addWidget(self.page)
        self.page_2 = QtWidgets.QWidget()
        self.page_2.setObjectName("page_2")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.page_2)
        self.horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_3.setSpacing(0)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.nameOut = QtWidgets.QCheckBox(self.page_2)
        self.nameOut.setCheckable(True)
        self.nameOut.setChecked(False)
        self.nameOut.setTristate(False)
        self.nameOut.setObjectName("nameOut")
        self.horizontalLayout_3.addWidget(self.nameOut)
        spacerItem1 = QtWidgets.QSpacerItem(136, 26, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem1)
        self.stack.addWidget(self.page_2)
        self.page_3 = QtWidgets.QWidget()
        self.page_3.setObjectName("page_3")
        self.gridLayout = QtWidgets.QGridLayout(self.page_3)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setObjectName("gridLayout")
        self.slider = QtWidgets.QSlider(self.page_3)
        self.slider.setEnabled(False)
        self.slider.setMaximum(65535)
        self.slider.setOrientation(QtCore.Qt.Horizontal)
        self.slider.setObjectName("slider")
        self.gridLayout.addWidget(self.slider, 0, 0, 1, 1)
        self.count = QtWidgets.QLCDNumber(self.page_3)
        self.count.setFrameShape(QtWidgets.QFrame.NoFrame)
        self.count.setNumDigits(5)
        self.count.setDigitCount(5)
        self.count.setSegmentStyle(QtWidgets.QLCDNumber.Flat)
        self.count.setObjectName("count")
        self.gridLayout.addWidget(self.count, 0, 1, 1, 1)
        self.thresholdSlider = QtWidgets.QSlider(self.page_3)
        self.thresholdSlider.setMaximum(65535)
        self.thresholdSlider.setProperty("value", 1000)
        self.thresholdSlider.setOrientation(QtCore.Qt.Horizontal)
        self.thresholdSlider.setObjectName("thresholdSlider")
        self.gridLayout.addWidget(self.thresholdSlider, 1, 0, 1, 1)
        self.thresLabel = QtWidgets.QLabel(self.page_3)
        self.thresLabel.setObjectName("thresLabel")
        self.gridLayout.addWidget(self.thresLabel, 1, 1, 1, 1)
        self.stack.addWidget(self.page_3)
        self.horizontalLayout.addWidget(self.stack)
        self.comboBox = QtWidgets.QComboBox(Frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comboBox.sizePolicy().hasHeightForWidth())
        self.comboBox.setSizePolicy(sizePolicy)
        self.comboBox.setMaximumSize(QtCore.QSize(85, 16777215))
        self.comboBox.setLayoutDirection(QtCore.Qt.RightToLeft)
        self.comboBox.setMaxCount(10)
        self.comboBox.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToContentsOnFirstShow)
        self.comboBox.setIconSize(QtCore.QSize(16, 16))
        self.comboBox.setFrame(False)
        self.comboBox.setObjectName("comboBox")
        self.comboBox.addItem("")
        self.comboBox.addItem("")
        self.comboBox.addItem("")
        self.horizontalLayout.addWidget(self.comboBox)

        self.retranslateUi(Frame)
        self.stack.setCurrentIndex(0)
        self.comboBox.setCurrentIndex(0)
        self.nameOut.clicked['bool'].connect(Frame.setOutputState) # type: ignore
        self.pullup.clicked['bool'].connect(Frame.setOutputState) # type: ignore
        self.comboBox.currentIndexChanged['int'].connect(Frame.setPage) # type: ignore
        self.slider.valueChanged['int'].connect(self.count.display) # type: ignore
        self.thresholdSlider.valueChanged['int'].connect(Frame.setThreshold) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(Frame)

    def retranslateUi(self, Frame):
        _translate = QtCore.QCoreApplication.translate
        Frame.setWindowTitle(_translate("Frame", "Frame"))
        self.nameIn.setText(_translate("Frame", "name"))
        self.pullup.setText(_translate("Frame", "Pull-Up"))
        self.pullup.setProperty("class", _translate("Frame", "pullup"))
        self.nameOut.setText(_translate("Frame", "name"))
        self.thresLabel.setToolTip(_translate("Frame", "The GREEN LED attached to PD5 will toggle when the counter crosses this"))
        self.thresLabel.setText(_translate("Frame", "PD5⇆"))
        self.comboBox.setItemText(0, _translate("Frame", "    INPUT"))
        self.comboBox.setItemText(1, _translate("Frame", "OUTPUT"))
        self.comboBox.setItemText(2, _translate("Frame", "     CNTR"))
